import sys

DEFAULT_CONFIG = "./sltx-config.yml"
C_DRIVER_LOG = "driver_log"
C_TEX_HOME = "tex_home"
C_WORKING_DIR = "working_dir"
C_CREATE_DIRS = "create_dirs"
C_CLEANUP = "cleanup"
C_AUTODETECT_DRIVERS = "autodetect_drivers"
C_DRIVERS = "drivers"
C_DRIVER_PATTERNS = "driver_patterns"
C_RECURSIVE = "recursive"

configuration = {
    C_TEX_HOME: "~/texmf/tex/latex/sltxinst",
    C_WORKING_DIR: "~/.sltxinst",
    C_DRIVER_LOG: "sltx-drivers.log",
    C_CREATE_DIRS: True,
    C_CLEANUP: True,
    C_RECURSIVE: True,
    C_AUTODETECT_DRIVERS: True,
    # TODO maybe specific install routine instead of plain copy
    C_DRIVERS: {
        "git": {
            "command": "git clone --depth 1 {args} \"{url}\" \"{working_dir}/{dep_name}\"",
            "target-dir": "{working_dir}/{dep_name}",
            "needs-delete": True  # if already exists
            # TODO: maybe update routine?
        }
        # TODO: other
    },
    C_DRIVER_PATTERNS: {
        "git": ["github", "gitlab"]
    }
}

dependencies = {}

def print_idx(idx : int, *objects, sep=' ', end='\n', file=sys.stdout, flush=False):
    print("[ID " + str(idx) + "]", *objects, sep=sep, end=end, file=file, flush=flush)

# global arguments read in
args = None


import argparse
import os  # list directory
import shutil  # clean working dir
import sys  # cmd line args
from concurrent import futures

from sltxpkg import dep, generate, globals as sg
from sltxpkg.config import (assure_dirs, load_configuration,
                         load_dependencies_config, write_to_log)
from sltxpkg.dep import install_dependencies
from sltxpkg.globals import (C_AUTODETECT_DRIVERS, C_CLEANUP, C_CREATE_DIRS,
                          C_DRIVER_LOG, C_DRIVER_PATTERNS, C_DRIVERS,
                          C_TEX_HOME, C_WORKING_DIR, DEFAULT_CONFIG, C_RECURSIVE)


def valid_file(arg: str):
    if arg is None or arg.strip() == "":
        raise ValueError("arg vas none or empty")
    if not os.path.isfile(arg):
        raise FileNotFoundError("\"" + arg + "\" must be an existing file")
    return arg


parser = argparse.ArgumentParser(
    description="sltx, a Simple LaTeX-INSTaller utility", prog="sltx")
parser.add_argument('-g', '--generate', dest='gen', action='store_true',
                    help="generate a github action file for automated installation.")
parser.add_argument('-c', '--config', dest='config', metavar='config.yml',
                    required=False, type=valid_file,
                    help="the file to load the configuration from.")
parser.add_argument('-d', '--dependencies', dest='dep', metavar='dep.yml',
                    required=False, type=valid_file,
                    help="the file to load the dependencies from.")
parser.add_argument('-t', '--threads', metavar='N', dest='threads', type=int,
                    help="number of threads to run the installation. Default is 1.",
                    default=1)

if(len(sys.argv) <= 1):
    parser.parse_args(['-h'])

sg.args = parser.parse_args()

if sg.args.gen:
    generate.generate()
    exit(0)

# TODO: if no deps or no generate call
if os.path.isfile(DEFAULT_CONFIG):
    print("Automatically loading '{DEFAULT_CONFIG}'".format(**locals()))
    load_configuration(DEFAULT_CONFIG)


if sg.args.config is not None:
    load_configuration(sg.args.config)
if sg.args.dep is not None:
    sg.dependencies = load_dependencies_config(sg.args.dep, sg.dependencies)

assure_dirs()

if "target" not in sg.dependencies or "dependencies" not in sg.dependencies:
    print("The dependency-file must supply a 'target' and an 'dependencies' key!")
    sys.exit(1)

write_to_log("====Dependencies for:" + sg.dependencies["target"]+"\n")
print()
print("Dependencies for:", sg.dependencies["target"])
print("Installing to:", sg.configuration[C_TEX_HOME])
print()

install_dependencies(0, sg.dependencies, first=True)

# all installed
if sg.configuration[C_CLEANUP]:
    print("> Cleaning up the working directory, as set.")
    shutil.rmtree(sg.configuration[C_WORKING_DIR])
print("Loaded:", dep.loaded)
if not sg.configuration[C_RECURSIVE]:
    print("Recursion was disabled.")
print("Dependency installation for",
      sg.dependencies["target"], "completed.")



from sys import platform


def default_texmf():
    if platform == "linux" or platform == "linux2":
        return "~/texmf"
    elif platform == "darwin":
        return "~/Library/texmf"
    elif platform == "win32":
        return "~/texmf"
